@@ -11,6 +11,7 @@ from _mysql_exceptions import Warning, Error, InterfaceError, DataError, \
      DatabaseError, OperationalError, IntegrityError, InternalError, \
      NotSupportedError, ProgrammingError
 import types, _mysql
+import re
 
 
 def defaulterrorhandler(connection, cursor, errorclass, errorvalue):
@@ -34,6 +35,23 @@ def defaulterrorhandler(connection, cursor, errorclass, errorvalue):
     del connection
     raise errorclass, errorvalue
 
+re_numeric_part = re.compile(r"^(\d+)")
+
+def numeric_part(s):
+    """Returns the leading numeric part of a string.
+    
+    >>> numeric_part("20-alpha")
+    20
+    >>> numeric_part("foo")
+    >>> numeric_part("16b")
+    16
+    """
+    
+    m = re_numeric_part.match(s)
+    if m:
+        return int(m.group(1))
+    return None
+
 
 class Connection(_mysql.connection):
 
@@ -140,11 +158,10 @@ class Connection(_mysql.connection):
 
         conv2 = {}
         for k, v in conv.items():
-            if isinstance(k, int):
-                if isinstance(v, list):
-                    conv2[k] = v[:]
-                else:
-                    conv2[k] = v
+            if isinstance(k, int) and isinstance(v, list):
+                conv2[k] = v[:]
+            else:
+                conv2[k] = v
         kwargs2['conv'] = conv2
 
         self.cursorclass = kwargs2.pop('cursorclass', self.default_cursor)
@@ -159,7 +176,7 @@ class Connection(_mysql.connection):
         sql_mode = kwargs2.pop('sql_mode', '')
 
         client_flag = kwargs.get('client_flag', 0)
-        client_version = tuple([ int(n) for n in _mysql.get_client_info().split('.')[:2] ])
+        client_version = tuple([ numeric_part(n) for n in _mysql.get_client_info().split('.')[:2] ])
         if client_version >= (4, 1):
             client_flag |= CLIENT.MULTI_STATEMENTS
         if client_version >= (5, 0):
@@ -172,7 +189,7 @@ class Connection(_mysql.connection):
         self.encoders = dict([ (k, v) for k, v in conv.items()
                                if type(k) is not int ])
         
-        self._server_version = tuple([ int(n) for n in self.get_server_info().split('.')[:2] ])
+        self._server_version = tuple([ numeric_part(n) for n in self.get_server_info().split('.')[:2] ])
 
         db = proxy(self)
         def _get_string_literal():
@@ -277,7 +294,7 @@ class Connection(_mysql.connection):
                 super(Connection, self).set_character_set(charset)
             except AttributeError:
                 if self._server_version < (4, 1):
-                    raise NotSupportedError, "server is too old to set charset"
+                    raise NotSupportedError("server is too old to set charset")
                 self.query('SET NAMES %s' % charset)
                 self.store_result()
         self.string_decoder.charset = charset
@@ -287,7 +304,7 @@ class Connection(_mysql.connection):
         """Set the connection sql_mode. See MySQL documentation for
         legal values."""
         if self._server_version < (4, 1):
-            raise NotSupportedError, "server is too old to set sql_mode"
+            raise NotSupportedError("server is too old to set sql_mode")
         self.query("SET SESSION sql_mode='%s'" % sql_mode)
         self.store_result()
         
